@@ -3,7 +3,7 @@
 #
 # debian/deps/pqxx.sh - this file is part of Gerbera.
 #
-# Copyright (C) 2025 Gerbera Contributors
+# Copyright (C) 2025-2026 Gerbera Contributors
 #
 # Gerbera is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2
@@ -20,4 +20,4 @@
 # $Id$
 
 sudo apt-get install -y \
-    libpq-dev
+    libpq-dev libpq5
