@@ -4,7 +4,7 @@
 
     config_setup_autoscan.cc - this file is part of Gerbera.
 
-    Copyright (C) 2020-2025 Gerbera Contributors
+    Copyright (C) 2020-2026 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -80,7 +80,7 @@ bool ConfigAutoscanSetup::createOptionFromNode(
         config->registerNode(element.path());
     }
     for (auto&& it : cs->getXmlTree(element)) {
-        const pugi::xml_node& child = it.node();
+        auto child = it.node();
         if (config) {
             config->registerNode(child.path());
         }
