@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2025 Gerbera Contributors
+    Copyright (C) 2016-2026 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -29,7 +29,7 @@
     $Id$
 */
 
-/// \file main.cc
+/// @file main.cc
 
 /// \mainpage Sourcecode Documentation.
 ///
@@ -204,13 +204,20 @@ int main(int argc, char** argv, char** envp)
         (GRB_OPTION_COMPILEINFO, "Print compile info and exit") //
         (GRB_OPTION_CREATECONFIG, "Print a default config.xml file and exit", cxxopts::value<std::string>()->implicit_value("All"), "Section") //
         (GRB_OPTION_CREATEEXAMPLECONFIG, "Print a example config.xml file and exit", cxxopts::value<std::string>()->implicit_value("All"), "Section") //
+        (GRB_OPTION_CREATEADVANCEDCONFIG, "Print a advanced example config.xml file and exit", cxxopts::value<std::string>()->implicit_value("All"), "Section") //
         (GRB_OPTION_CHECKCONFIG, "Check config.xml file and exit") //
         (GRB_OPTION_PRINTOPTIONS, "Print simple config options and exit") //
-        (GRB_OPTION_OFFLINE, "Do not answer UPnP content requests") // good for initial scans
+        (GRB_OPTION_OFFLINE, "Do not answer UPnP content requests", cxxopts::value<bool>()->default_value("false")) // good for initial scans
+        (GRB_OPTION_DROPTABLES, "Drop all database tables and exit", cxxopts::value<bool>()->default_value("false")) //
+#ifdef HAVE_LASTFM
+#ifndef HAVE_LASTFMLIB
+        (GRB_OPTION_INITLASTFM, "Get Last.FM session key", cxxopts::value<bool>()->default_value("false")) //
+#endif
+#endif
         ("d," GRB_OPTION_DAEMON, "Daemonize after startup", cxxopts::value<bool>()->default_value("false")) //
         ("u," GRB_OPTION_USER, "Drop privs to user UID", cxxopts::value<std::string>(), "UID") //
         ("P," GRB_OPTION_PIDFILE, "Write a pidfile to the specified location, e.g. /run/gerbera.pid", cxxopts::value<fs::path>(), "FILE") //
-        ("c," GRB_OPTION_CONFIG, "Path to config file", cxxopts::value<fs::path>(), "DIR") //
+        ("c," GRB_OPTION_CONFIG, "Path to config file", cxxopts::value<fs::path>(), "FILE") //
         ("f," GRB_OPTION_CFGDIR, "Override name of config folder (.config/gerbera) in home folder", cxxopts::value<fs::path>(), "DIR") //
         ("l," GRB_OPTION_LOGFILE, "Set log location", cxxopts::value<fs::path>(), "FILE") //
         (GRB_OPTION_ROTATELOG, "Set rotating log location", cxxopts::value<fs::path>(), "FILE") //
@@ -234,6 +241,7 @@ int main(int argc, char** argv, char** envp)
         std::shared_ptr<ConfigManager> configManager;
         int portnum = -1;
         try {
+            runtime.notifySystemd(ServiceStatus::Reloading);
             configManager = std::make_shared<ConfigManager>(
                 definition,
                 runtime.getConfigFile(), runtime.getHome(), runtime.getConfDir(),
@@ -266,8 +274,17 @@ int main(int argc, char** argv, char** envp)
         auto server = std::make_shared<Server>(configManager);
 
         try {
-            server->init(definition, runtime.getOffline());
-            server->run();
+            auto dropDatabase = runtime.getDropDatabase();
+            auto initLastFM = runtime.getLastFM();
+
+            server->init(definition, runtime.getOffline(), dropDatabase, initLastFM);
+
+            if (!dropDatabase && !initLastFM)
+                server->run();
+            else
+                runtime.exit(EXIT_SUCCESS);
+
+            runtime.notifySystemd(ServiceStatus::Ready);
         } catch (const UpnpException& ue) {
             log_error("{}", ue.what());
 
@@ -292,6 +309,7 @@ int main(int argc, char** argv, char** envp)
             }
 
             try {
+                runtime.notifySystemd(ServiceStatus::Stopping);
                 if (server)
                     server->shutdown();
                 server.reset();
@@ -321,6 +339,7 @@ int main(int argc, char** argv, char** envp)
 
             if (_ctx.restartFlag != 0) {
                 log_info("Restarting Gerbera!");
+                runtime.notifySystemd(ServiceStatus::Reloading);
                 try {
                     server->shutdown();
                     server.reset();
@@ -362,8 +381,9 @@ int main(int argc, char** argv, char** envp)
                     installSignalHandler();
 
                     server = std::make_shared<Server>(configManager);
-                    server->init(definition, runtimeChild.getOffline());
+                    server->init(definition, runtimeChild.getOffline(), runtimeChild.getDropDatabase(), runtimeChild.getLastFM());
                     server->run();
+                    runtime.notifySystemd(ServiceStatus::Ready);
 
                     _ctx.restartFlag = 0;
                 } catch (const std::runtime_error& ex) {
@@ -387,6 +407,7 @@ int main(int argc, char** argv, char** envp)
         // shutting down
         int ret = EXIT_SUCCESS;
         try {
+            runtime.notifySystemd(ServiceStatus::Stopping);
             server->shutdown();
             server.reset();
             configManager.reset();
