@@ -4,7 +4,7 @@
 
     test_common_script.cc - this file is part of Gerbera.
 
-    Copyright (C) 2020-2025 Gerbera Contributors
+    Copyright (C) 2020-2026 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -30,30 +30,10 @@
 
 #include <duktape.h>
 
-class CommonScriptTest : public ::testing::Test {
+class CommonScriptTest : public ScriptTestFixture {
 
 public:
-    CommonScriptTest() = default;
-    ~CommonScriptTest() override = default;
-
-    void SetUp() override
-    {
-        ctx = duk_create_heap(nullptr, nullptr, nullptr, nullptr, nullptr);
-
-        fs::path scriptPath = fs::path(SCRIPTS_DIR) / "js" / "common.js";
-        std::string script = GrbFile(scriptPath).readTextFile();
-        duk_push_string(ctx, scriptPath.c_str());
-        if (duk_pcompile_lstring_filename(ctx, 0, script.c_str(), script.length()) != 0) {
-            DukTestHelper::printError(ctx, "Failed to load script ", scriptPath);
-        } else {
-            duk_call(ctx, 0);
-        }
-    }
-
-    void TearDown() override
-    {
-        duk_destroy_heap(ctx);
-    }
+    CommonScriptTest() { }
 
     static std::string invokeABCBOX(duk_context* ctx, const std::string& input, int boxType, const std::string& divChar)
     {
@@ -78,8 +58,6 @@ public:
         duk_pcall(ctx, 3);
         return duk_to_string(ctx, -1);
     }
-
-    duk_context* ctx;
 };
 
 TEST_F(CommonScriptTest, CreatesDukContextWithCommonScript)
@@ -99,6 +77,172 @@ TEST_F(CommonScriptTest, escapeSlash_AddsEscapeCharsForSlash)
     EXPECT_STREQ(result.c_str(), "some\\/path\\/to\\/escape");
 }
 
+TEST_F(CommonScriptTest, weekOf0)
+{
+    duk_get_global_string(ctx, "weekOf");
+    duk_push_int(ctx, 2024);
+    duk_push_int(ctx, 0); // month starts at 0
+    duk_push_int(ctx, 1);
+
+    duk_pcall(ctx, 3);
+
+    std::string result = duk_to_string(ctx, -1);
+
+    EXPECT_STREQ(result.c_str(), "2024-01") << "2024-01-01";
+
+    duk_get_global_string(ctx, "weekOf");
+    duk_push_int(ctx, 2023);
+    duk_push_int(ctx, 11); // month starts at 0
+    duk_push_int(ctx, 31);
+
+    duk_pcall(ctx, 3);
+
+    result = duk_to_string(ctx, -1);
+
+    EXPECT_STREQ(result.c_str(), "2023-52") << "2023-12-31";
+
+    duk_get_global_string(ctx, "weekOf");
+    duk_push_int(ctx, 2024);
+    duk_push_int(ctx, 0); // month starts at 0
+    duk_push_int(ctx, 7);
+
+    duk_pcall(ctx, 3);
+
+    result = duk_to_string(ctx, -1);
+
+    EXPECT_STREQ(result.c_str(), "2024-01") << "2024-01-07";
+}
+
+TEST_F(CommonScriptTest, weekOf1)
+{
+    duk_get_global_string(ctx, "weekOf");
+    duk_push_int(ctx, 2025);
+    duk_push_int(ctx, 0); // month starts at 0
+    duk_push_int(ctx, 1);
+
+    duk_pcall(ctx, 3);
+
+    std::string result = duk_to_string(ctx, -1);
+
+    EXPECT_STREQ(result.c_str(), "2025-01") << "2025-01-01";
+
+    duk_get_global_string(ctx, "weekOf");
+    duk_push_int(ctx, 2024);
+    duk_push_int(ctx, 11); // month starts at 0
+    duk_push_int(ctx, 30);
+
+    duk_pcall(ctx, 3);
+
+    result = duk_to_string(ctx, -1);
+
+    EXPECT_STREQ(result.c_str(), "2025-01") << "2024-12-30";
+
+    duk_get_global_string(ctx, "weekOf");
+    duk_push_int(ctx, 2025);
+    duk_push_int(ctx, 0); // month starts at 0
+    duk_push_int(ctx, 5);
+
+    duk_pcall(ctx, 3);
+
+    result = duk_to_string(ctx, -1);
+
+    EXPECT_STREQ(result.c_str(), "2025-01") << "2025-01-05";
+}
+
+TEST_F(CommonScriptTest, weekOf53)
+{
+    duk_get_global_string(ctx, "weekOf");
+    duk_push_int(ctx, 2027);
+    duk_push_int(ctx, 0); // month starts at 0
+    duk_push_int(ctx, 1);
+
+    duk_pcall(ctx, 3);
+
+    std::string result = duk_to_string(ctx, -1);
+
+    EXPECT_STREQ(result.c_str(), "2026-53") << "2027-01-01";
+
+    duk_get_global_string(ctx, "weekOf");
+    duk_push_int(ctx, 2026);
+    duk_push_int(ctx, 11); // month starts at 0
+    duk_push_int(ctx, 27);
+
+    duk_pcall(ctx, 3);
+
+    result = duk_to_string(ctx, -1);
+
+    EXPECT_STREQ(result.c_str(), "2026-52") << "2026-12-27";
+
+    duk_get_global_string(ctx, "weekOf");
+    duk_push_int(ctx, 2026);
+    duk_push_int(ctx, 11); // month starts at 0
+    duk_push_int(ctx, 28);
+
+    duk_pcall(ctx, 3);
+
+    result = duk_to_string(ctx, -1);
+
+    EXPECT_STREQ(result.c_str(), "2026-53") << "2026-12-28";
+
+    duk_get_global_string(ctx, "weekOf");
+    duk_push_int(ctx, 2027);
+    duk_push_int(ctx, 0); // month starts at 0
+    duk_push_int(ctx, 3);
+
+    duk_pcall(ctx, 3);
+
+    result = duk_to_string(ctx, -1);
+
+    EXPECT_STREQ(result.c_str(), "2026-53") << "2027-01-03";
+
+    duk_get_global_string(ctx, "weekOf");
+    duk_push_int(ctx, 2027);
+    duk_push_int(ctx, 0); // month starts at 0
+    duk_push_int(ctx, 4);
+
+    duk_pcall(ctx, 3);
+
+    result = duk_to_string(ctx, -1);
+
+    EXPECT_STREQ(result.c_str(), "2027-01") << "2027-01-04";
+}
+
+TEST_F(CommonScriptTest, weekOf45)
+{
+    duk_get_global_string(ctx, "weekOf");
+    duk_push_int(ctx, 2025);
+    duk_push_int(ctx, 10); // month starts at 0
+    duk_push_int(ctx, 3);
+
+    duk_pcall(ctx, 3);
+
+    std::string result = duk_to_string(ctx, -1);
+
+    EXPECT_STREQ(result.c_str(), "2025-45");
+
+    duk_get_global_string(ctx, "weekOf");
+    duk_push_int(ctx, 2025);
+    duk_push_int(ctx, 10); // month starts at 0
+    duk_push_int(ctx, 2);
+
+    duk_pcall(ctx, 3);
+
+    result = duk_to_string(ctx, -1);
+
+    EXPECT_STREQ(result.c_str(), "2025-44");
+
+    duk_get_global_string(ctx, "weekOf");
+    duk_push_int(ctx, 2025);
+    duk_push_int(ctx, 10); // month starts at 0
+    duk_push_int(ctx, 8);
+
+    duk_pcall(ctx, 3);
+
+    result = duk_to_string(ctx, -1);
+
+    EXPECT_STREQ(result.c_str(), "2025-45");
+}
+
 TEST_F(CommonScriptTest, mapInitial_Latin)
 {
     duk_get_global_string(ctx, "mapInitial");
